@@ -28,7 +28,11 @@ from ...model import ProductType
 try:
     import importlib.resources as importlib_resources
 except ImportError:
-    import importlib_resources
+    import importlib_resources  # type: ignore
+
+if typing.TYPE_CHECKING:
+    from ...types import TypeClassifier
+    from ...model import Cluster, Gene
 
 
 class Run(Annotate):  # noqa: D101
@@ -36,12 +40,12 @@ class Run(Annotate):  # noqa: D101
     summary = "predict BGC from one or several contigs."
 
     @classmethod
-    def doc(cls, fast=False):  # noqa: D102
+    def doc(cls, fast: bool = False) -> str:  # noqa: D102
         return f"""
         gecco run - {cls.summary}
 
         Usage:
-            gecco run --genome <file> [--hmm <hmm>]... [options]
+            gecco run --genome <file> [--hmm <hmm>]... [--hmm-x <hmm>]... [options]
 
         Arguments:
             -g <file>, --genome <file>    a genomic file containing one or more
@@ -67,11 +71,17 @@ class Run(Annotate):  # noqa: D101
                                           when they are empty (e.g. because
                                           no genes or no clusters were found).
 
-
         Parameters - Gene Calling:
             -M, --mask                    Enable unknown region masking to
                                           prevent genes from stretching across
                                           unknown nucleotides.
+            --cds-feature <cds_feature>   Extract genes from annotated records
+                                          using a feature rather than calling
+                                          genes from scratch.
+            --locus-tag <locus_tag>       The name of the feature qualifier
+                                          to use for naming extracted genes
+                                          when using the ``--cds-feature``
+                                          flag. [default: locus_tag]
 
         Parameters - Domain Annotation:
             -e <e>, --e-filter <e>        the e-value cutoff for protein domains
@@ -104,11 +114,13 @@ class Run(Annotate):  # noqa: D101
         Parameters - Debug:
             --model <directory>           the path to an alternative CRF model
                                           to use (obtained with `gecco train`).
-            --hmm <hmm>                   the path to one or more alternative HMM
-                                          file to use (in HMMER format).
+            --hmm <hmm>                   the path to one or more alternative
+                                          HMM file to use (in HMMER format).
+            --hmm-x <hmm>                 the path to one or more exclusive
+                                          HMM file to use (in HMMER format).
         """
 
-    def _check(self) -> typing.Optional[int]:
+    def _check(self) -> None:
         Command._check(self)
         try:
             self.cds = self._check_flag("--cds", int, lambda x: x > 0, hint="positive integer")
@@ -135,12 +147,15 @@ class Run(Annotate):  # noqa: D101
             self.edge_distance = self._check_flag("--edge-distance", int, lambda x: x >= 0, hint="positive or null integer")
             self.format = self._check_flag("--format", optional=True)
             self.genome = self._check_flag("--genome")
-            self.model = self._check_flag("--model", optional=True)
+            self.model: Optional[str] = self._check_flag("--model", optional=True)
             self.hmm = self._check_flag("--hmm")
+            self.hmm_x = self._check_flag("--hmm-x")
             self.output_dir = self._check_flag("--output-dir")
             self.antismash_sideload = self._check_flag("--antismash-sideload", bool)
             self.force_tsv = self._check_flag("--force-tsv", bool)
             self.mask = self._check_flag("--mask", bool)
+            self.cds_feature = self._check_flag("--cds-feature", optional=True)
+            self.locus_tag = self._check_flag("--locus-tag")
             self.no_pad = self._check_flag("--no-pad", bool)
         except InvalidArgument:
             raise CommandExit(1)
@@ -160,12 +175,12 @@ class Run(Annotate):  # noqa: D101
         except FileNotFoundError as err:
             if self.model is not None:
                 self.error("Could not find domains list :", repr(self.model))
-            raise CommandExit(e.errno) from err
+            raise CommandExit(err.errno) from err
         else:
             self.success("Found", len(domains), "selected features", level=2)
             return domains
 
-    def _predict_probabilities(self, genes):
+    def _predict_probabilities(self, genes: List["Gene"]) -> List["Gene"]:
         from ...crf import ClusterCRF
 
         if self.model is None:
@@ -182,7 +197,7 @@ class Run(Annotate):  # noqa: D101
             pad=not self.no_pad,
         ))
 
-    def _extract_clusters(self, genes):
+    def _extract_clusters(self, genes: List["Gene"]) -> List["Cluster"]:
         from ...refine import ClusterRefiner
 
         self.info("Extracting", "predicted biosynthetic regions", level=1)
@@ -197,39 +212,43 @@ class Run(Annotate):  # noqa: D101
         unit = "contigs" if total > 1 else "contig"
         task = self.progress.add_task("Extracting clusters", total=total, unit=unit, precision="")
 
-        clusters = []
-        gene_groups = itertools.groupby(genes, lambda g: g.source.id)
+        clusters: List["Cluster"] = []
+        gene_groups = itertools.groupby(genes, lambda g: g.source.id)  # type: ignore
         for _, gene_group in self.progress.track(gene_groups, task_id=task, total=total):
-            clusters.extend(refiner.iter_clusters(gene_group))
+            clusters.extend(refiner.iter_clusters(list(gene_group)))
 
         return clusters
 
-    def _predict_types(self, clusters):
-        from ...model import ProductType
+    def _load_type_classifier(self) -> "TypeClassifier":
         from ...types import TypeClassifier
 
+        self.info("Loading", "type classifier from internal model", level=2)
+        return TypeClassifier.trained(self.model)
+
+    def _predict_types(self, clusters: List["Cluster"], classifier: "TypeClassifier") -> List["Cluster"]:
+        from ...model import ProductType
+
         self.info("Predicting", "BGC types", level=1)
 
         unit = "cluster" if len(clusters) == 1 else "clusters"
         task = self.progress.add_task("Predicting types", total=len(clusters), unit=unit, precision="")
 
         clusters_new = []
-        classifier = TypeClassifier.trained(self.model)
         for cluster in self.progress.track(clusters, task_id=task):
             clusters_new.extend(classifier.predict_types([cluster]))
-            if cluster.type != ProductType.Unknown:
-                name = "/".join(f"[bold blue]{ty.name}[/]" for ty in cluster.type.unpack())
-                prob = "/".join(f"[bold purple]{cluster.type_probabilities[ty]:.0%}[/]" for ty in cluster.type.unpack())
+            if cluster.type:
+                name = "/".join(f"[bold blue]{name}[/]" for name in cluster.type.names)
+                prob = "/".join(f"[bold purple]{cluster.type_probabilities[name]:.0%}[/]" for name in cluster.type.names)
                 self.success(f"Predicted type of [bold blue]{cluster.id}[/] as {name} ({prob} confidence)")
             else:
-                ty = max(cluster.type_probabilities, key=cluster.type_probabilities.get)
+                ty = max(cluster.type_probabilities, key=cluster.type_probabilities.get)   # type: ignore
                 prob = f"[bold purple]{cluster.type_probabilities[ty]:.0%}[/]"
-                name = f"[bold blue]{ty.name}[/]"
+                name = f"[bold blue]{ty}[/]"
                 self.warn(f"Couldn't assign type to [bold blue]{cluster.id}[/] (maybe {name}, {prob} confidence)")
 
         return clusters_new
 
-    def _write_cluster_table(self, clusters):
+    def _write_cluster_table(self, clusters: List["Cluster"]) -> None:
         from ...model import ClusterTable
 
         base, _ = os.path.splitext(os.path.basename(self.genome))
@@ -238,7 +257,7 @@ class Run(Annotate):  # noqa: D101
         with open(cluster_out, "w") as out:
             ClusterTable.from_clusters(clusters).dump(out)
 
-    def _write_clusters(self, clusters):
+    def _write_clusters(self, clusters: List["Cluster"]) -> None:
         from Bio import SeqIO
 
         for cluster in clusters:
@@ -246,9 +265,9 @@ class Run(Annotate):  # noqa: D101
             self.info("Writing", f"cluster [bold blue]{cluster.id}[/] to", repr(gbk_out), level=1)
             SeqIO.write(cluster.to_seq_record(), gbk_out, "genbank")
 
-    def _write_sideload_json(self, clusters):
+    def _write_sideload_json(self, clusters: List["Cluster"]) -> None:
         # record version and important parameters
-        data = {
+        data: Dict[str, Any] = {
             "records": [],
             "tool": {
                 "name": "GECCO",
@@ -270,23 +289,21 @@ class Run(Annotate):  # noqa: D101
         if self.model:
             data["tool"]["configuration"]["model"] = self.model
         # create a record per sequence
-        for seq_id, seq_clusters in itertools.groupby(clusters, key=lambda cluster: cluster.source.id):
+        for seq_id, seq_clusters in itertools.groupby(clusters, key=operator.attrgetter("source.id")):
             data["records"].append({"name": seq_id, "subregions": []})
             for cluster in seq_clusters:
-                ty = ";".join(sorted(ty.name for ty in cluster.type.unpack())) or "Unknown"
+                probabilities = {
+                    f"{key.lower()}_probability":f"{value:.3f}"
+                    for key, value in cluster.type_probabilities.items()
+                }
                 data["records"][-1]["subregions"].append({
                     "start": cluster.start,
                     "end": cluster.end,
-                    "label": ty,
+                    "label": ";".join(sorted(cluster.type.names)) or "Unknown",
                     "details": {
                         "average_p": f"{cluster.average_probability:.3f}",
                         "max_p": f"{cluster.maximum_probability:.3f}",
-                        "alkaloid_probability": f"{cluster.type_probabilities.get(ProductType.Alkaloid, 0.0):.3f}",
-                        "polyketide_probability": f"{cluster.type_probabilities.get(ProductType.Polyketide, 0.0):.3f}",
-                        "ripp_probability": f"{cluster.type_probabilities.get(ProductType.RiPP, 0.0):.3f}",
-                        "saccharide_probability": f"{cluster.type_probabilities.get(ProductType.Saccharide, 0.0):.3f}",
-                        "terpene_probability": f"{cluster.type_probabilities.get(ProductType.Terpene, 0.0):.3f}",
-                        "nrp_probability": f"{cluster.type_probabilities.get(ProductType.NRP, 0.0):.3f}",
+                        **probabilities,
                     }
                 })
         # write the JSON file to the output folder
@@ -303,7 +320,7 @@ class Run(Annotate):  # noqa: D101
             # check the CLI arguments were fine and enter context
             self._check()
             ctx.enter_context(self.progress)
-            ctx.enter_context(patch_showwarnings(self._showwarnings))
+            ctx.enter_context(patch_showwarnings(self._showwarnings))  # type: ignore
             # attempt to create the output directory, checking it doesn't
             # already contain output files (or raise a warning)
             extensions = ["features.tsv", "genes.tsv", "clusters.tsv"]
@@ -340,7 +357,9 @@ class Run(Annotate):  # noqa: D101
                     self._write_cluster_table(clusters)
                 return 0
             # predict types for putative clusters
-            clusters = self._predict_types(clusters)
+            classifier = self._load_type_classifier()
+            if len(classifier.classes_) > 1:
+                clusters = self._predict_types(clusters, classifier)
             # write results
             self.info("Writing", "result files to folder", repr(self.output_dir), level=1)
             self._write_cluster_table(clusters)
