@@ -24,9 +24,12 @@ from Bio.SeqFeature import SeqFeature, FeatureLocation, CompoundLocation, Refere
 from Bio.SeqRecord import SeqRecord
 
 from . import __version__
-from ._base import Dumpable, Table
+from ._base import Dumpable, Table, _SELF
 from ._meta import patch_locale
 
+if typing.TYPE_CHECKING:
+    from numpy.typing import NDArray
+
 
 __all__ = [
     "ProductType",
@@ -41,40 +44,45 @@ __all__ = [
 
 
 # fmt: off
-class ProductType(enum.IntFlag):
-    """A flag to declare the type of product synthesized by a gene cluster.
+class ProductType(object):
+    """An immutable storage of type for the product synthesized by a cluster.
     """
 
-    Unknown    = 0b00000000
-    Alkaloid   = 0b00000010
-    Polyketide = 0b00000100
-    RiPP       = 0b00001000
-    Saccharide = 0b00010000
-    Terpene    = 0b00100000
-    NRP        = 0b01000000
-
-    @classmethod
-    def pack(cls, members: Iterable["ProductType"]) -> "ProductType":
-        """Pack together a list of individual product types.
+    def __init__(self, *names: str) -> None:
+        """Create a new product type from one or more base types.
 
         Example:
-            >>> types = [ProductType.Polyketide, ProductType.Saccharide]
-            >>> ProductType.pack(types)
-            <ProductType.Saccharide|Polyketide: 20>
+            >>> t1 = ProductType()                    # unknown type
+            >>> t2 = ProductType("Polyketide")        # single type
+            >>> t3 = ProductType("Polyketide", "NRP") # multiple types
 
         """
-        return functools.reduce(operator.or_, members, cls.Unknown)
+        self.names = frozenset(names)
+
+    def __repr__(self) -> str:  # noqa: D105
+        return "ProductType({})".format(", ".join(map(repr, sorted(self.names))))
+
+    def __hash__(self) -> int:  # noqa: D105
+        return hash(self.names)
+
+    def __eq__(self, other: object) -> bool:  # noqa: D105
+        if not isinstance(other, ProductType):
+            return NotImplemented
+        return self.names == other.names
+
+    def __bool__(self) -> bool:  # noqa: D105
+        return len(self.names) != 0
 
     def unpack(self) -> List["ProductType"]:
         """Unpack a composite `ProductType` into a list of individual types.
 
         Example:
-            >>> ty = ProductType.Polyketide | ProductType.Saccharide
+            >>> ty = ProductType("Polyketide", "Saccharide")
             >>> ty.unpack()
-            [<ProductType.Polyketide: 4>, <ProductType.Saccharide: 16>]
+            [ProductType('Polyketide'), ProductType('Saccharide')]
 
         """
-        return [ x for x in ProductType.__members__.values() if (x & self) ]
+        return [ ProductType(x) for x in sorted(self.names) ]
 
 
 class Strand(enum.IntEnum):
@@ -122,7 +130,7 @@ class Domain:
     i_evalue: float
     pvalue: float
     probability: Optional[float] = None
-    qualifiers: Mapping[str, Union[str, List[str]]] = field(default_factory=dict)
+    qualifiers: Dict[str, List[str]] = field(default_factory=dict)
 
     def with_probability(self, probability: Optional[float]) -> "Domain":
         """Copy the current domain and assign it a BGC probability.
@@ -144,7 +152,7 @@ class Domain:
         stride = 1 if protein_coordinates else 3
         loc = FeatureLocation((self.start-1)*stride, self.end*stride)
         qualifiers = dict(self.qualifiers)
-        qualifiers.setdefault("standard_name", self.name)
+        qualifiers.setdefault("standard_name", [self.name])
         return SeqFeature(location=loc, type="misc_feature", qualifiers=qualifiers)
 
 
@@ -209,7 +217,7 @@ class Gene:
     end: int
     strand: Strand
     protein: Protein
-    qualifiers: Mapping[str, Union[str, List[str]]] = field(default_factory=dict)
+    qualifiers: Dict[str, List[str]] = field(default_factory=dict)
     _probability: Optional[float] = field(default_factory=lambda: None)
 
     @property
@@ -227,10 +235,6 @@ class Gene:
         p = [d.probability for d in self.protein.domains if d.probability is not None]
         return sum(p) / len(p) if p else None
 
-    @average_probability.setter
-    def average_probability(self, probability: Optional[float]):
-        self._probability = probability
-
     @property
     def maximum_probability(self) -> Optional[float]:
         """`float`: The highest of domain probabilities of being biosynthetic.
@@ -249,8 +253,8 @@ class Gene:
         # but Biopython expects 0-based ranges with exclusive ends
         loc = FeatureLocation(start=self.start, end=self.end+1, strand=int(self.strand))
         qualifiers = dict(self.qualifiers)
-        qualifiers.setdefault("locus_tag", self.protein.id)
-        qualifiers.setdefault("translation", str(self.protein.seq))
+        qualifiers.setdefault("locus_tag", [self.protein.id])
+        qualifiers.setdefault("translation", [str(self.protein.seq)])
         return SeqFeature(location=loc, type="CDS", qualifiers=qualifiers)
 
     def with_protein(self, protein: "Protein") -> "Gene":
@@ -305,25 +309,20 @@ class Cluster:
     id: str
     genes: List[Gene]
     type: ProductType
-    type_probabilities: Mapping[ProductType, float]
+    type_probabilities: Dict[str, float]
 
     def __init__(
         self,
         id: str,
         genes: Optional[List[Gene]] = None,
-        type: ProductType = ProductType.Unknown,
-        type_probabilities: Optional[Dict[ProductType, float]] = None,
+        type: ProductType = ProductType(),
+        type_probabilities: Optional[Dict[str, float]] = None,
     ):  # noqa: D107
         self.id = id
         self.genes = genes or list()
         self.type = type
         self.type_probabilities = type_probabilities or dict()
 
-        # if len(self.types) != len(self.types_probabilities):
-        #     err = "type and type probability lists must have the same dimensions"
-        #     raise ValueError(err)
-
-
     @property
     def source(self) -> SeqRecord:  # type: ignore
         """`~Bio.SeqRecord.SeqRecord`: The sequence this cluster was found in.
@@ -364,7 +363,7 @@ class Cluster:
         normalize: bool = True,
         minlog_weights: bool = False,
         pvalue: bool = True,
-    ) -> numpy.ndarray:
+    ) -> "NDArray[numpy.double]":
         """Compute weighted domain composition with respect to ``all_possible``.
 
         Arguments:
@@ -402,7 +401,7 @@ class Cluster:
             if dom in unique_names:
                 composition[i] = numpy.sum(weights[names == dom])
         if normalize:
-            return composition / (composition.sum() or 1)  # type: ignore
+            return composition / (composition.sum() or 1)
         return composition
 
     # ---
@@ -454,17 +453,16 @@ class Cluster:
         bgc.annotations.setdefault("references", []).append(ref)
 
         # add GECCO-specific annotations as a structured comment
+        probabilities = {
+            f"{key.lower()}_probability":f"{value:.3f}"
+            for key, value in self.type_probabilities.items()
+        }
         structured_comment = bgc.annotations.setdefault("structured_comment", OrderedDict())
         structured_comment['GECCO-Data'] = {
             "version": f"GECCO v{__version__}",
             "creation_date": now.isoformat(),
-            "biosyn_class": ";".join(sorted(ty.name for ty in self.type.unpack())) or "Unknown",
-            "alkaloid_probability": self.type_probabilities.get(ProductType.Alkaloid, 0.0),
-            "polyketide_probability": self.type_probabilities.get(ProductType.Polyketide, 0.0),
-            "ripp_probability": self.type_probabilities.get(ProductType.RiPP, 0.0),
-            "saccharide_probability": self.type_probabilities.get(ProductType.Saccharide, 0.0),
-            "terpene_probability": self.type_probabilities.get(ProductType.Terpene, 0.0),
-            "nrp_probability": self.type_probabilities.get(ProductType.NRP, 0.0),
+            "biosyn_class": ";".join(sorted(self.type.names)) or "Unknown",
+            **probabilities,
         }
 
         # add proteins as CDS features
@@ -541,6 +539,22 @@ class FeatureTable(Table):
         domain_end: int
         bgc_probability: Optional[float]
 
+    if typing.TYPE_CHECKING:
+
+        @typing.overload  # type: ignore
+        def __getitem__(self, item: int) -> "FeatureTable.Row":  # noqa: D105
+            pass
+
+        @typing.overload
+        def __getitem__(self, item: slice) -> "FeatureTable.Row":  # noqa: D105
+            pass
+
+        def __getitem__(self, item: Union[slice, int]) -> Union["FeatureTable", "FeatureTable.Row"]:   # noqa: D105
+            return super().__getitem__(item)  # type: ignore
+
+        def __iter__(self) -> Iterator["FeatureTable.Row"]:  # type: ignore  # noqa: D105
+            return super().__iter__()  # type: ignore
+
     @classmethod
     def from_genes(cls, genes: Iterable[Gene]) -> "FeatureTable":
         """Create a new feature table from an iterable of genes.
@@ -594,13 +608,11 @@ class FeatureTable(Table):
 
 
 def _format_product_type(value: "ProductType") -> str:
-    types = value.unpack() or [ProductType.Unknown]
-    return ";".join(sorted(map(operator.attrgetter("name"), types)))
+    return ";".join(sorted(value.names))
 
 
 def _parse_product_type(value: str) -> "ProductType":
-    types = [ProductType.__members__[x] for x in value.split(";")]
-    return ProductType.pack(types)
+    return ProductType(*map(str.strip, value.split(";"))) if value.strip() else ProductType()
 
 
 @dataclass(frozen=True)
@@ -610,18 +622,13 @@ class ClusterTable(Table):
 
     sequence_id: List[str] = field(default_factory = list)
     bgc_id: List[str] = field(default_factory = list)
-    start: List[int] = field(default_factory = lambda: array("l"))          # type: ignore
-    end: List[int] = field(default_factory = lambda: array("l"))            # type: ignore
-    average_p: List[Optional[float]] = field(default_factory = list)    # type: ignore
-    max_p: List[Optional[float]] = field(default_factory = list)        # type: ignore
+    start: List[int] = field(default_factory = lambda: array("l"))   # type: ignore
+    end: List[int] = field(default_factory = lambda: array("l"))     # type: ignore
+    average_p: List[Optional[float]] = field(default_factory = list)
+    max_p: List[Optional[float]] = field(default_factory = list)
 
     type: List[ProductType] = field(default_factory = list)
-    alkaloid_probability: List[Optional[float]] = field(default_factory = list)    # type: ignore
-    polyketide_probability: List[Optional[float]] = field(default_factory = list)  # type: ignore
-    ripp_probability: List[Optional[float]] = field(default_factory = list)        # type: ignore
-    saccharide_probability: List[Optional[float]] = field(default_factory = list)  # type: ignore
-    terpene_probability: List[Optional[float]] = field(default_factory = list)     # type: ignore
-    nrp_probability: List[Optional[float]] = field(default_factory = list)         # type: ignore
+    type_p: List[Dict[str, float]] = field(default_factory = list)
 
     proteins: List[Optional[List[str]]] = field(default_factory = list)
     domains: List[Optional[List[str]]] = field(default_factory = list)
@@ -636,13 +643,10 @@ class ClusterTable(Table):
         end: int
         average_p: Optional[float]
         max_p: Optional[float]
+
         type: ProductType
-        alkaloid_probability: Optional[float]
-        polyketide_probability: Optional[float]
-        ripp_probability: Optional[float]
-        saccharide_probability: Optional[float]
-        terpene_probability: Optional[float]
-        nrp_probability: Optional[float]
+        type_p: Dict[ProductType, float]
+
         proteins: Optional[List[str]]
         domains: Optional[List[str]]
 
@@ -663,12 +667,7 @@ class ClusterTable(Table):
             table.max_p.append(cluster.maximum_probability)
 
             table.type.append(cluster.type)
-            table.alkaloid_probability.append(cluster.type_probabilities.get(ProductType.Alkaloid, 0))
-            table.polyketide_probability.append(cluster.type_probabilities.get(ProductType.Polyketide, 0))
-            table.ripp_probability.append(cluster.type_probabilities.get(ProductType.RiPP, 0))
-            table.saccharide_probability.append(cluster.type_probabilities.get(ProductType.Saccharide, 0))
-            table.terpene_probability.append(cluster.type_probabilities.get(ProductType.Terpene, 0))
-            table.nrp_probability.append(cluster.type_probabilities.get(ProductType.NRP, 0))
+            table.type_p.append(cluster.type_probabilities.copy())
 
             table.proteins.append([ gene.protein.id for gene in cluster.genes ])
             domains = {d.name for g in cluster.genes for d in g.protein.domains}
@@ -678,6 +677,94 @@ class ClusterTable(Table):
     def __len__(self) -> int:  # noqa: D105
         return len(self.sequence_id)
 
+    if typing.TYPE_CHECKING:
+
+        @typing.overload  # type: ignore
+        def __getitem__(self, item: int) -> "ClusterTable.Row":  # noqa: D105
+            pass
+
+        @typing.overload
+        def __getitem__(self, item: slice) -> "ClusterTable.Row":  # noqa: D105
+            pass
+
+        def __getitem__(self, item: Union[slice, int]) -> Union["ClusterTable", "ClusterTable.Row"]:   # noqa: D105
+            return super().__getitem__(item)  # type: ignore
+
+        def __iter__(self) -> Iterator["ClusterTable.Row"]:  # type: ignore  # noqa: D105
+            return super().__iter__()  # type: ignore
+
+    def dump(self, fh: TextIO, dialect: str = "excel-tab", header: bool = True) -> None:  # noqa: D102
+        writer = csv.writer(fh, dialect=dialect)
+        column_names = list(self.__annotations__)
+        type_p_col = column_names.index("type_p")
+        column_names.pop(type_p_col)
+        optional = self._optional_columns()
+
+        # do not write optional columns if they are completely empty
+        for name in optional:
+            if all(x is None for x in getattr(self, name)):
+                column_names.remove(name)
+
+        # build column formatters
+        columns = [getattr(self, name) for name in column_names]
+        formatters = [self._FORMAT_FIELD[self.Row.__annotations__[name]] for name in column_names]
+
+        # add probability columns if any row countains probabilities
+        if any(d for d in self.type_p):
+            classes = sorted({
+                name
+                for probabilities in self.type_p
+                for name in probabilities.keys()
+            })
+            for name in classes:
+                column_names.insert(type_p_col, f"{name.lower()}_probability")
+                columns.insert(type_p_col, [self.type_p[i][name] for i in range(len(self))])
+                formatters.insert(type_p_col, str)  # type: ignore
+                type_p_col += 1
+
+         # write header if desired
+        if header:
+            writer.writerow(column_names)
+        # write each row
+        for i in range(len(self)):
+            writer.writerow([format(col[i]) for col,format in zip(columns, formatters)])
+
+    @classmethod
+    def load(cls, fh: TextIO, dialect: str = "excel-tab") -> "ClusterTable":  # noqa: D102
+        table = cls()
+        reader = csv.reader(fh, dialect=dialect)
+        header = next(reader)
+
+        # check that if a column is missing, it is one of the optional values
+        optional = cls._optional_columns()
+        missing = set(cls.__annotations__).difference({"type_p"}).difference(header)
+        missing_required = missing.difference(optional)
+        if missing_required:
+            raise ValueError("table is missing columns: {}".format(", ".join(missing_required)))
+
+        # get the name of each column and check which columns are optional
+        columns = [getattr(table, col, None) for col in header]
+        parsers = [
+            cls._PARSE_FIELD[table.Row.__annotations__[col]]
+            if col in table.Row.__annotations__
+            else None
+            for col in header
+        ]
+
+        # extract elements from the CSV rows
+        for row in reader:
+            probabilities = {}
+            for name, col, value, parser in zip(header, columns, row, parsers):
+                if parser is not None and col is not None:
+                    col.append(parser(value))
+                elif name.endswith("_probability"):
+                    probabilities[name[:-12]] = float(value)
+            table.type_p.append(probabilities)
+
+        for col in missing:
+            getattr(table, col).extend(None for _ in range(len(table)))
+        return table
+
 
 @dataclass(frozen=True)
 class GeneTable(Table):
@@ -689,8 +776,8 @@ class GeneTable(Table):
     start: List[int] = field(default_factory = lambda: array("l"))    # type: ignore
     end: List[int] = field(default_factory = lambda: array("l"))      # type: ignore
     strand: List[str] = field(default_factory = list)
-    average_p: List[Optional[float]] = field(default_factory = list)  # type: ignore
-    max_p: List[Optional[float]] = field(default_factory = list)      # type: ignore
+    average_p: List[Optional[float]] = field(default_factory = list)
+    max_p: List[Optional[float]] = field(default_factory = list)
 
     class Row(NamedTuple):
         """A single row in a gene table.
@@ -704,6 +791,22 @@ class GeneTable(Table):
         average_p: Optional[float]
         max_p: Optional[float]
 
+    if typing.TYPE_CHECKING:
+
+        @typing.overload  # type: ignore
+        def __getitem__(self, item: int) -> "GeneTable.Row":  # noqa: D105
+            pass
+
+        @typing.overload
+        def __getitem__(self, item: slice) -> "GeneTable.Row":  # noqa: D105
+            pass
+
+        def __getitem__(self, item: Union[slice, int]) -> Union["GeneTable", "GeneTable.Row"]:   # noqa: D105
+            return super().__getitem__(item)  # type: ignore
+
+        def __iter__(self) -> Iterator["GeneTable.Row"]:  # type: ignore  # noqa: D105
+            return super().__iter__()  # type: ignore
+
     @classmethod
     def from_genes(cls, genes: Iterable[Gene]) -> "GeneTable":
         """Create a new gene table from an iterable of genes.
@@ -727,7 +830,7 @@ class GeneTable(Table):
         be converted to a `~Bio.SeqRecord.SeqRecord` if needed.
 
         """
-        for row in self:
+        for row in typing.cast(Iterable["GeneTable.Row"], self):
             source = SeqRecord(id=row.sequence_id, seq=_UnknownSeq())
             strand = Strand.Coding if row.strand == "+" else Strand.Reverse
             seq = Seq("X" * (row.end - row.start // 3))
